grammar calc2;

@header {
import java.lang.Double;
import java.lang.Math;
}

@members {
    public String getErrorMessage(RecognitionException e, String[] tokenNames) {
        List stack = getRuleInvocationStack(e, this.getClass().getName());
        String msg = null;
        if (e instanceof NoViableAltException) {
            NoViableAltException nvae = (NoViableAltException)e;
            msg = " no viable alt; token = " + e.token + 
                  " (decision = " + nvae.decisionNumber + 
                  " state " + nvae.stateNumber + ")" +
                  " decision = <<" + nvae.grammarDecisionDescription + ">>";
        } else {
            msg = super.getErrorMessage(e, tokenNames);
        }
        return stack + " " + msg;
    }

    public String getTokenErrorDisplay(Token t) {
        return t.toString();
    }
}

calc
	: lines EOF
	;
	
lines
	: (line NL)+
	;

line
	: expr {System.out.println($expr.value);}
	;

expr returns [double value]
	: t1=term {$value = $t1.value;} (PLUS t2=term {$value += $t2.value;} | MINUS t2=term {$value -= $t2.value;})*
	;

term returns [double value]
	: f1=factor {$value = $f1.value;} (MULT f2=factor {$value *= $f2.value;} | DIV f2=factor {$value /= $f2.value;})*
	;
	
factor returns [double value]
	: PLUS f1=factor {$value = $f1.value;}
	| MINUS f2=factor {$value = -$f2.value;}
	| power {$value = $power.value;}
	;

power returns [double value]
	: atom {$value = $atom.value;} (POW f=factor {$value = Math.pow($value, $f.value);})?
	;
	
atom returns [double value]
	: DOUBLE {$value = Double.parseDouble($DOUBLE.text);}
	| PI {$value = Math.PI;}
	| E {$value = Math.E;}
	| LPAR expr RPAR {$value = $expr.value;}
	;

PI
	: 'PI'
	;

E
	: 'E'
	;
	
DOUBLE
    : INT
    | '.' INT
    | INT '.'
    | INT '.' INT
    ;

fragment INT
	: '0'..'9'+
	;

PLUS
	: '+'
	;
	
MINUS
	: '-'
	;
	
MULT
	: '*'
	;
	
DIV
	: '/'
	;
	
POW
	: '**'
	;

LPAR
    : '('
    ;

RPAR
    : ')'
    ;

NL
	: '\r'|'\n'
	;

WHITESPACE
	: ('\t'|' '|'\u000C')+ {$channel = HIDDEN;}
	;
